#!/usr/bin/env python
# coding: utf-8
import pdb
import sys
import re
import configparser
import numpy as np
import smtk.trellis.trellis_plots as trpl
import smtk.trellis.configure as rcfg
from openquake.baselib import sap
from openquake.hazardlib.gsim import get_available_gsims
from matplotlib import pyplot
import warnings; warnings.filterwarnings("ignore")

def str_to_list(tmps):
    return re.split(',', re.sub('\\s*', '', re.sub('\\[|\\]', '', tmps)))

def get_error(ini_fname):
    available_gsims = get_available_gsims()
    config = configparser.ConfigParser()
    config.read(ini_fname)
    gmpe_list = str_to_list(config['general']['gmpe_list'])
    for gmpe in gmpe_list:
        gsim = available_gsims[gmpe]()
        sas = []
        attrs = dir(gsim)
        sub = 'coeffs'
        coeffs = [s for s in attrs if sub.lower() in s.lower()]
        for co in coeffs:
            print("Check your IMTs! ",gsim," can only produce the following periods of SA!")
            print(getattr(gsim, co).sa_coeffs.keys())

def plot_by_distance(ini_fname):

    #read in config file
    config = configparser.ConfigParser()
    config.read(ini_fname)
    
    #specify GMPEs
    gmpe_list = str_to_list(config['general']['gmpe_list'])

    #specify IMTs
    imt_list = str_to_list(config['general']['imt_list'])

    #specify the rupture
    params = {"ztor": float(config['rupture']['ztor']),   # Top of rupture depth (km)
         "hypo_depth": float(config['rupture']['hypo_depth']),   # Hypocentral depth (km)
         "vs30": float(config['rupture']['vs30']), # Vs30 for all sites
         "vs30measured": True, # Vs30 value is measured, currently not configurable
         "width": float(config['rupture']['width']), # width (km) of rupture
         "z1pt0": float(config['rupture']['z1pt0']), # Depth (m) to the 1.0 km/s Vs interface
         "z2pt5": float(config['rupture']['z2pt5']), # Depth (km) to the 2.5 km/s Vs interface 
         "dip": float(config['rupture']['dip']),  
         "rake": float(config['rupture']['rake']), # -90=Normal fault
         "backarc": True} #currenlty not configurable

    #set the magnitude
    magnitude = float(config['rupture']['magnitude'])
    
    #set distances
    distances = {"repi": np.arange(0.1, 200, 1.0)} # distances from 0.1km to 200km
    distances["rhypo"] = np.sqrt(distances["repi"] ** 2.0 + params["hypo_depth"] ** 2)
    distances["rjb"] = distances["repi"] # approximation; does not consider that ruptures are extended in space
    distances["rrup"] = np.sqrt(distances["rjb"] ** 2.0 + params["ztor"] ** 2)
    distances["rx"] = distances["rjb"] # approximation; does not consider that ruptures are extended in space

    #decide which type of distance to use
    available_gsims = get_available_gsims()
    dctx=[]
    if len(gmpe_list)>1:
        for gmpe in gmpe_list:
            chkng = available_gsims[gmpe]
            dctx.append(chkng.REQUIRES_DISTANCES)
        sub = 'rjb'
        cf = [s for s in dctx if sub in s]
        if len(cf)>0:            
            dist_imt_trellis = trpl.DistanceIMTTrellis(magnitude, distances, gmpe_list, imt_list, params,
                                          figure_size=(8,6), ylim=(0.001,10), filename='plot.png')
        else:
            cffh2 = str(dctx[0]).split("'")[1]
            dist_imt_trellis = trpl.DistanceIMTTrellis(magnitude, distances, gmpe_list, imt_list, params,
                                              distance_type=cffh2, figure_size=(8,6), ylim=(0.001,10), filename='plot.png')
    else:
        chkng = available_gsims[gmpe_list[0]]
        cffh2 = str(chkng.REQUIRES_DISTANCES).split("'")[1]
        dist_imt_trellis = trpl.DistanceIMTTrellis(magnitude, distances, gmpe_list, imt_list, params,
                                              distance_type=cffh2, figure_size=(8,6), ylim=(0.001,10), filename='plot.png')
    
  
    dist_imt_trellis.plot()


def main(argv):
    try:
        p = sap.Script(plot_by_distance)

        msg = 'Path to the configuration fname - typically a .ini file'
        p.arg(name='ini_fname', help=msg) 

        if len(argv) < 1:
            print(p.help())
        else:
            p.callfunc()

    except IndexError as error: 
            #get  list of sa+pga 
        p2 = sap.Script(get_error)
        msg = 'Path to the configuration fname - typically a .ini file'
        p2.arg(name='ini_fname', help=msg)
        p2.callfunc()

if __name__ == "__main__":
    main(sys.argv[1:])
